@@ -39,10 +39,44 @@
 #define CONFIG_NUM 1
 #endif
 
-#define CDC_IF_STR_IX (MYNEWT_VAL(USBD_CDC_DESCRIPTOR_STRING) == NULL ? 0 : 4)
-#define MSC_IF_STR_IX (MYNEWT_VAL(USBD_MSC_DESCRIPTOR_STRING) == NULL ? 0 : 5)
-#define HID_IF_STR_IX (MYNEWT_VAL(USBD_HID_DESCRIPTOR_STRING) == NULL ? 0 : 6)
-#define BTH_IF_STR_IX (MYNEWT_VAL(USBD_BTH_DESCRIPTOR_STRING) == NULL ? 0 : 7)
+enum usb_desc_ix {
+    USB_DESC_IX_SERIAL_NUMBER   = 1,
+    USB_DESC_IX_VENDOR          = 2,
+    USB_DESC_IX_PRODUCT         = 3,
+#if defined MYNEWT_VAL_USBD_CDC_DESCRIPTOR_STRING
+    CDC_IF_STR_IX,
+#else
+#define CDC_IF_STR_IX 0
+#endif
+#if defined MYNEWT_VAL_USBD_CDC_CONSOLE_DESCRIPTOR_STRING
+    CDC_CONSOLE_IF_STR_IX,
+#else
+#define CDC_CONSOLE_IF_STR_IX   0
+#endif
+#if defined MYNEWT_VAL_USBD_CDC_HCI_DESCRIPTOR_STRING
+    CDC_HCI_IF_STR_IX,
+#else
+#define CDC_HCI_IF_STR_IX   0
+#endif
+#if defined MYNEWT_VAL_USBD_MSC_DESCRIPTOR_STRING
+    MSC_IF_STR_IX,
+#else
+#define MSC_IF_STR_IX   0
+#endif
+#if defined MYNEWT_VAL_USBD_HID_DESCRIPTOR_STRING
+    HID_IF_STR_IX,
+#else
+#define HID_IF_STR_IX   0
+#endif
+#if defined MYNEWT_VAL_USBD_BTH_DESCRIPTOR_STRING
+    BTH_IF_STR_IX,
+#else
+#define BTH_IF_STR_IX   0
+#endif
+#if defined MYNEWT_VAL_USBD_DFU_SLOT_NAME
+    DFU_SLOT_NAME_IF_STR_IX,
+#endif
+};
 
 #if MYNEWT_VAL(USBD_CONFIGURATION_SELF_POWERED)
 #define SELF_POWERED_OPT    TUSB_DESC_CONFIG_ATT_SELF_POWERED
@@ -58,6 +92,16 @@
 
 #define CONFIG_ATT          (SELF_POWERED_OPT | REMOTE_WAKEUP_OPT)
 
+#if defined(MYNEWT_VAL_USBD_DFU_SLOT_NAME)
+#define USBD_DFU_SLOT_NAME  MYNEWT_VAL(USBD_DFU_SLOT_NAME)
+#else
+#define USBD_DFU_SLOT_NAME  NULL
+#endif
+
+#ifndef CFG_TUD_DFU_DETACH_TIMEOUT
+#define CFG_TUD_DFU_DETACH_TIMEOUT  1000
+#endif
+
 #if CFG_TUD_HID
 
 #if MYNEWT_VAL(USBD_HID_REPORT_ID_KEYBOARD) || MYNEWT_VAL(USBD_HID_REPORT_ID_MOUSE)
@@ -133,23 +177,9 @@ const tusb_desc_device_t desc_device = {
     .bDescriptorType    = TUSB_DESC_DEVICE,
     .bcdUSB             = USB_BCD,
 
-#if CFG_TUD_BTH
-    .bDeviceClass       = TUD_BT_APP_CLASS,
-    .bDeviceSubClass    = TUD_BT_APP_SUBCLASS,
-    .bDeviceProtocol    = TUD_BT_PROTOCOL_PRIMARY_CONTROLLER,
-#elif CFG_TUD_CDC
-    /*
-     * Use Interface Association Descriptor (IAD) for CDC
-     * As required by USB Specs IAD's subclass must be common class (2) and protocol must be IAD (1)
-     */
-    .bDeviceClass       = TUSB_CLASS_MISC,
-    .bDeviceSubClass    = MISC_SUBCLASS_COMMON,
-    .bDeviceProtocol    = MISC_PROTOCOL_IAD,
-#else
-    .bDeviceClass       = 0x00,
-    .bDeviceSubClass    = 0x00,
-    .bDeviceProtocol    = 0x00,
-#endif
+    .bDeviceClass       = MYNEWT_VAL(USBD_DEVICE_CLASS),
+    .bDeviceSubClass    = MYNEWT_VAL(USBD_DEVICE_SUBCLASS),
+    .bDeviceProtocol    = MYNEWT_VAL(USBD_DEVICE_PROTOCOL),
 
     .bMaxPacketSize0    = CFG_TUD_ENDPOINT0_SIZE,
 
@@ -233,11 +263,21 @@ enum {
 #endif
 #endif
 
-#if CFG_TUD_CDC
+#if CFG_CDC
     ITF_NUM_CDC,
     ITF_NUM_CDC_DATA,
 #endif
 
+#if CFG_CDC_CONSOLE
+    ITF_NUM_CDC_CONSOLE,
+    ITF_NUM_CDC_CONSOLE_DATA,
+#endif
+
+#if CFG_CDC_HCI
+    ITF_NUM_CDC_HCI,
+    ITF_NUM_CDC_HCI_DATA,
+#endif
+
 #if CFG_TUD_MSC
     ITF_NUM_MSC,
 #endif
@@ -254,7 +294,9 @@ enum {
 };
 
 #define CONFIG_TOTAL_LEN    (TUD_CONFIG_DESC_LEN + \
-                             CFG_TUD_CDC * TUD_CDC_DESC_LEN + \
+                             CFG_CDC * TUD_CDC_DESC_LEN + \
+                             CFG_CDC_CONSOLE * TUD_CDC_DESC_LEN + \
+                             CFG_CDC_HCI * TUD_CDC_DESC_LEN + \
                              CFG_TUD_MSC * TUD_MSC_DESC_LEN + \
                              CFG_TUD_HID * TUD_HID_DESC_LEN + \
                              CFG_TUD_BTH * TUD_BTH_DESC_LEN + \
@@ -272,7 +314,19 @@ const uint8_t desc_configuration[] = {
                        0, 9, 17, 25, 33, 49),
 #endif
 
-#if CFG_TUD_CDC
+#if CFG_CDC_CONSOLE
+    TUD_CDC_DESCRIPTOR(ITF_NUM_CDC_CONSOLE, CDC_CONSOLE_IF_STR_IX, USBD_CDC_CONSOLE_NOTIFY_EP,
+                       USBD_CDC_CONSOLE_NOTIFY_EP_SIZE, USBD_CDC_CONSOLE_DATA_OUT_EP, USBD_CDC_CONSOLE_DATA_IN_EP,
+                       (CFG_TUSB_RHPORT0_MODE & OPT_MODE_HIGH_SPEED) ? 512 : USBD_CDC_CONSOLE_DATA_EP_SIZE),
+#endif
+
+#if CFG_CDC_HCI
+    TUD_CDC_DESCRIPTOR(ITF_NUM_CDC_HCI, CDC_HCI_IF_STR_IX, USBD_CDC_HCI_NOTIFY_EP, USBD_CDC_HCI_NOTIFY_EP_SIZE,
+                       USBD_CDC_HCI_DATA_OUT_EP, USBD_CDC_HCI_DATA_IN_EP,
+                       (CFG_TUSB_RHPORT0_MODE & OPT_MODE_HIGH_SPEED) ? 512 : USBD_CDC_HCI_DATA_EP_SIZE),
+#endif
+
+#if CFG_CDC
     TUD_CDC_DESCRIPTOR(ITF_NUM_CDC, CDC_IF_STR_IX, USBD_CDC_NOTIFY_EP, USBD_CDC_NOTIFY_EP_SIZE,
                        USBD_CDC_DATA_OUT_EP, USBD_CDC_DATA_IN_EP,
                        (CFG_TUSB_RHPORT0_MODE & OPT_MODE_HIGH_SPEED) ? 512 : USBD_CDC_DATA_EP_SIZE),
@@ -280,7 +334,7 @@ const uint8_t desc_configuration[] = {
 
 #if CFG_TUD_MSC
     /* TODO: MSC not handled yet */
-    TUD_MSC_DESCRIPTOR(ITF_NUM_MSC, MSC_IF_STR_IX, EPNUM_MSC_OUT, EPNUM_MSC_IN,
+    TUD_MSC_DESCRIPTOR(ITF_NUM_MSC, MSC_IF_STR_IX, USBD_MSC_DATA_OUT_EP, USBD_MSC_DATA_IN_EP,
                        (CFG_TUSB_RHPORT0_MODE & OPT_MODE_HIGH_SPEED) ? 512 : 64),
 #endif
 
@@ -290,7 +344,8 @@ const uint8_t desc_configuration[] = {
 #endif
 
 #if CFG_TUD_DFU
-    TUD_DFU_DESCRIPTOR(ITF_NUM_DFU, 1, 8, DFU_ATTR_CAN_DOWNLOAD, CFG_TUD_DFU_DETACH_TIMEOUT, CFG_TUD_DFU_XFER_BUFSIZE),
+    TUD_DFU_DESCRIPTOR(ITF_NUM_DFU, 1, DFU_SLOT_NAME_IF_STR_IX, DFU_ATTR_CAN_DOWNLOAD,
+                       CFG_TUD_DFU_DETACH_TIMEOUT, CFG_TUD_DFU_XFER_BUFSIZE),
 #endif
 };
 
@@ -310,14 +365,26 @@ tud_descriptor_configuration_cb(uint8_t index)
 const char *string_desc_arr[] = {
     MYNEWT_VAL(USBD_VENDOR_STRING),
     MYNEWT_VAL(USBD_PRODUCT_STRING),
+#if defined MYNEWT_VAL_USBD_CDC_DESCRIPTOR_STRING
     MYNEWT_VAL(USBD_CDC_DESCRIPTOR_STRING),
+#endif
+#if defined MYNEWT_VAL_USBD_CDC_CONSOLE_DESCRIPTOR_STRING
+    MYNEWT_VAL(USBD_CDC_CONSOLE_DESCRIPTOR_STRING),
+#endif
+#if defined MYNEWT_VAL_USBD_CDC_HCI_DESCRIPTOR_STRING
+    MYNEWT_VAL(USBD_CDC_HCI_DESCRIPTOR_STRING),
+#endif
+#if defined MYNEWT_VAL_USBD_MSC_DESCRIPTOR_STRING
     MYNEWT_VAL(USBD_MSC_DESCRIPTOR_STRING),
+#endif
+#if defined MYNEWT_VAL_USBD_HID_DESCRIPTOR_STRING
     MYNEWT_VAL(USBD_HID_DESCRIPTOR_STRING),
+#endif
+#if defined MYNEWT_VAL_USBD_BTH_DESCRIPTOR_STRING
     MYNEWT_VAL(USBD_BTH_DESCRIPTOR_STRING),
+#endif
 #if defined MYNEWT_VAL_USBD_DFU_SLOT_NAME
     MYNEWT_VAL(USBD_DFU_SLOT_NAME),
-#else
-    NULL,
 #endif
 };
 
@@ -400,14 +467,16 @@ tud_descriptor_string_cb(uint8_t index, uint16_t langid)
     } else if (index - 2 < ARRAY_SIZE(string_desc_arr)) {
         str = string_desc_arr[index - 2];
 
-        char_num = strlen(str);
-        assert(char_num <= ARRAY_SIZE(desc_string) - 1);
-        if (char_num > ARRAY_SIZE(desc_string) - 1) {
-            char_num = ARRAY_SIZE(desc_string) - 1;
-        }
+        if (str) {
+            char_num = strlen(str);
+            assert(char_num <= ARRAY_SIZE(desc_string) - 1);
+            if (char_num > ARRAY_SIZE(desc_string) - 1) {
+                char_num = ARRAY_SIZE(desc_string) - 1;
+            }
 
-        for (i = 0; i < char_num; ++i) {
-            desc_string[1 + i] = str[i];
+            for (i = 0; i < char_num; ++i) {
+                desc_string[1 + i] = str[i];
+            }
         }
     }
 
